package com.crm.manager.dao.mapper;

import com.crm.rely.backend.core.entity.account.deal.change.AccountDealPasswordChangeSearchEntity;
import com.crm.rely.backend.core.entity.cooperative.partner.agent.CooperativePartnerAgentSearchEntity;
import com.crm.rely.backend.core.entity.cooperative.partner.subset.CooperativePartnerSubsetSearchEntity;
import com.crm.rely.backend.core.entity.cooperative.partner.us.CooperativePartnerUsSearchEntity;
import com.crm.rely.backend.core.pojo.table.AccountDealPasswordChangeTable;
import com.crm.rely.backend.core.pojo.table.CooperativePartnerTable;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface CooperativePartnerMapper {

    Integer countAgentList(CooperativePartnerAgentSearchEntity entity);

    List<CooperativePartnerTable> pageAgentList(CooperativePartnerAgentSearchEntity entity);

    Integer countSubsetList(CooperativePartnerSubsetSearchEntity entity);

    List<CooperativePartnerTable> pageSubsetList(CooperativePartnerSubsetSearchEntity entity);

    Integer countUsList(CooperativePartnerUsSearchEntity entity);

    List<CooperativePartnerTable> pageUsList(CooperativePartnerUsSearchEntity entity);
}
